#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "exprlinkedstack.h"

int inStackPrecedence(Precedence oper);
int outStackPrecedence(Precedence oper);
void printToken(ExprToken element);

int inStackPrecedence(Precedence oper)
{
	switch (oper) {
	case lparen:
		return 0;
	case rparen:
		return 4;
	case multiply:
	case divide:
		return 2;
	case plus:
	case minus:
		return 1;
	}
	return -1;
}

int outStackPrecedence(Precedence oper)
{
	switch (oper) {
	case lparen:
		return 5;
	case rparen:
		return 4;
	case multiply:
	case divide:
		return 2;
	case plus:
	case minus:
		return 1;
	}
	return -1;
}

void printToken(ExprToken element)
{
	switch (element.type) {
	case lparen:
		printf("(\t");
		break;
	case rparen:
		printf(")\t");
		break;
	case plus:
		printf("+\t");
		break;
	case minus:
		printf("-\t");
		break;
	case multiply:
		printf("*\t");
		break;
	case divide:
		printf("/\t");
		break;
	case operand:
		printf("%4.1f\t", element.value);
		break;
	}
}

ExprToken *convertInfixToPostfix(ExprToken *pExprTokens, int tokenCount, int* pResultTokenCount)
{
	ExprToken *pResult = NULL;
	LinkedStack *pStack = NULL;
	LinkedStackNode *pNode = NULL;
	Precedence tokenType;
	Precedence inStackTokenType;
	int i = 0, current_token_index = 0;

	if (pExprTokens == NULL || tokenCount == 0) return NULL;
	
	pResult = (ExprToken *)malloc(sizeof(ExprToken) * tokenCount);
	if (pResult != NULL) {
		memset(pResult, 0, sizeof(ExprToken) * tokenCount);
	}
	else {
		printf(", ޸Ҵ,convertInfixToPostfix()\n");
		return NULL;
	}

	pStack = createLinkedStack();
	if (pStack != NULL) {
		for (i = 0; i < tokenCount; i++) {
			tokenType = pExprTokens[i].type;
			switch (tokenType) {
			case operand:
				//printf("%4.1f\t", pExprTokens[i].value);
				pResult[current_token_index].type = operand;
				pResult[current_token_index].value = pExprTokens[i].value;
				current_token_index++;
				break;
			case rparen:
				pNode = popLS(pStack);
				while (pNode != NULL && pNode->data.type != lparen) {
					//printToken(pNode->data);
					pResult[current_token_index].type = pNode->data.type;
					pResult[current_token_index].value = pNode->data.value;
					current_token_index++;

					free(pNode);

					pNode = popLS(pStack);
				}
				if (pNode != NULL) free(pNode);
				break;
			default:
				while (isLinkedStackEmpty(pStack) == 0) {
					inStackTokenType = peekLS(pStack)->data.type;
					if (outStackPrecedence(tokenType)
						<= inStackPrecedence(inStackTokenType)) {
						pNode = popLS(pStack);
						if (pNode != NULL) {
							//printToken(pNode->data);
							pResult[current_token_index].type = pNode->data.type;
							pResult[current_token_index].value = pNode->data.value;
							current_token_index++;

							free(pNode);
						}
					}
					else {
						break;
					}
				}
				pushLS(pStack, pExprTokens[i]);
				break;
			}//end-of-switch
		}//end-of-for

		while (isLinkedStackEmpty(pStack) == 0) {
			pNode = popLS(pStack);
			if (pNode != NULL) {
				//printToken(pNode->data);
				pResult[current_token_index].type = pNode->data.type;
				pResult[current_token_index].value = pNode->data.value;
				current_token_index++;

				free(pNode);
			}
		}
		deleteLinkedStack(pStack);
	}

	if (pResult != NULL && current_token_index > 0) {
		*pResultTokenCount = current_token_index;
	}

	return pResult;
}

void calcExpr(ExprToken *pExprTokens, int tokenCount)
{
	LinkedStack *pStack = NULL;
	LinkedStackNode *pNode1 = NULL, *pNode2 = NULL;
	Precedence tokenType;
	int i = 0;
	if (pExprTokens == NULL) {
		return;
	}

	pStack = createLinkedStack();
	if (pStack != NULL) {
		for (i = 0; i < tokenCount; i++) {
			tokenType = pExprTokens[i].type;

			if (tokenType == operand) {
				pushLS(pStack, pExprTokens[i]);
			}
			else {
				pNode2 = popLS(pStack);
				if (pNode2 != NULL) {
					pNode1 = popLS(pStack);
					if (pNode1 != NULL) {
						double op1 = pNode1->data.value;
						double op2 = pNode2->data.value;

						ExprToken newToken;
						newToken.type = operand;
						switch (tokenType) {
						case plus:
							newToken.value = op1 + op2;
							break;
						case minus:
							newToken.value = op1 - op2;
							break;
						case multiply:
							newToken.value = op1 * op2;
							break;
						case divide:
							newToken.value = op1 / op2;
							break;
						}
						pushLS(pStack, newToken);
						free(pNode1);
					}
					free(pNode2);
				}
			} //end-of-else
		}

		pNode1 = popLS(pStack);
		if (pNode1 != NULL) {
			printf("   [%f]Դϴ\n",
				pNode1->data.value);
			free(pNode1);
		}
		else {
			printf(",Ŀ  ֽϴ\n");
		}
		deleteLinkedStack(pStack);
	}
}


int main(int argc, const char * argv[]) 
{
	int i = 0, num_index = 0, current_token = 0;
	ExprToken *pPostfixTokens = NULL;
	int result_token_count = 0;
	ExprToken tokens[1024] = { 0 };
	char strToken[1024] = { 0 };
	char strExpression[1024]; // = { 0 };
	printf(" Էϼ, ) ( 10 + 9 ) * 8 / 2 \n");
	fgets(strExpression, sizeof(strExpression), stdin);

	if (strExpression[0] == 0) {
		printf(" Է ʾҽϴ\n");
		return 0;
	}
	while (strExpression[i] != 0) {
		if (strExpression[i] >= (int)'0' && strExpression[i] <= (int)'9') {
			strToken[num_index] = strExpression[i];
			num_index++;
		}
		else {
			if (num_index > 0) {
				tokens[current_token].type = operand;
				tokens[current_token].value = atof(strToken);
				memset(strToken, 0, sizeof(strToken));
				num_index = 0;
				current_token++;
			}

			switch (strExpression[i]) {
			case '+':
				tokens[current_token].type = plus;
				tokens[current_token].value = 0;
				current_token++;
				break;
			case '-':
				tokens[current_token].type = minus;
				tokens[current_token].value = 0;
				current_token++;
				break;
			case '*':
				tokens[current_token].type = multiply;
				tokens[current_token].value = 0;
				current_token++;
				break;
			case '/':
				tokens[current_token].type = divide;
				tokens[current_token].value = 0;
				current_token++;
				break;
			case '(':
				tokens[current_token].type = lparen;
				tokens[current_token].value = 0;
				current_token++;
				break;
			case ')':
				tokens[current_token].type = rparen;
				tokens[current_token].value = 0;
				current_token++;
				break;
			}
		}
		i++;
	}
	if (num_index > 0) {
		tokens[current_token].type = operand;
		tokens[current_token].value = atof(strToken);
		memset(strToken, 0, sizeof(strToken));
		num_index = 0;
		current_token++;
	}

	pPostfixTokens = convertInfixToPostfix(tokens, current_token, &result_token_count);

	if (pPostfixTokens != NULL) {
		calcExpr(pPostfixTokens, result_token_count);
		free(pPostfixTokens);
	}

	return 0;
}
