#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "linkedlist.h"

#define LINKED_GRAPH_TYPE_DIRECT         0
#define LINKED_GRAPH_TYPE_UNDIRECT       1

typedef struct LinkedGraphType
{
    int graphType;    
    int nodeCount;          //  
    LinkedList** ppAdjEdge;	//     Ʈ () 迭
}  LinkedGraph;

LinkedGraph* createLinkedGraph(int graphType, int nodeCount)
{
    int i = 0;
    LinkedGraph *pReturn = NULL;
    
    if (nodeCount > 0)	{
        pReturn = ( LinkedGraph *)malloc(sizeof( LinkedGraph));
        if (pReturn == NULL) {
            printf(", ޸Ҵ(1), in createLinkedGraph()\n");
            return NULL;
        }
        pReturn->graphType = graphType;
        pReturn->nodeCount = nodeCount;
    }
    else {
        printf(", ִ   0 Ŀմϴ\n");
        return NULL;
    }
    
    pReturn->ppAdjEdge = (LinkedList **)malloc(sizeof(LinkedList*) * nodeCount);
    if (pReturn->ppAdjEdge == NULL) {
        printf(", ޸Ҵ(3), in createLinkedGraph()\n");
        if (pReturn != NULL)  free(pReturn);
        return NULL;
    }
    for(i = 0; i < nodeCount; i++) {
        pReturn->ppAdjEdge[i] = createLinkedList();
        if (pReturn->ppAdjEdge[i] == NULL) {
            printf(", ޸Ҵ(4), in createLinkedGraph()\n");
            if (pReturn->ppAdjEdge != NULL)  free(pReturn->ppAdjEdge);
            if (pReturn != NULL)  free(pReturn);
            return NULL;
        }
    }
    
    return pReturn;
}

int checkVertexValid(LinkedGraph* pGraph, int node)
{
    if (pGraph != NULL && node >= 0 && node < pGraph->nodeCount) {
        return 1;
    }
    else {
        return 0;
    }
}

int addEdgewithWeightInternalLG( LinkedGraph* pGraph, int fromNode, int toNode, int weight)
{
    int ret = 0;
    
    if (pGraph != NULL
        && checkVertexValid(pGraph, fromNode)
        && checkVertexValid(pGraph, toNode)) {
        addLinkedListData(pGraph->ppAdjEdge[fromNode], 0, toNode, weight);
    }
    else {
        ret = -1;
    }
    
    return ret;
}

int addEdgewithWeightLG( LinkedGraph* pGraph, int fromNode, int toNode, int weight)
{
    int ret = 0;
    
    ret = addEdgewithWeightInternalLG(pGraph, fromNode, toNode, weight);
    if (0 == ret && LINKED_GRAPH_TYPE_UNDIRECT == pGraph->graphType) {
        ret = addEdgewithWeightInternalLG(pGraph, toNode, fromNode, weight);
    }
    
    return ret;
}

int removeEdgeInternalLG(LinkedGraph* pGraph, int fromNode, int toNode)
{
    int ret = -1;
    LinkedList *pList = NULL;
    int i = 0, count = 0;
    
    if (pGraph != NULL
        && checkVertexValid(pGraph, fromNode)
        && checkVertexValid(pGraph, toNode)) {
        pList = pGraph->ppAdjEdge[fromNode];
        count = getLinkedListLength(pList);
        for(i = 0; i < count; i++) {
            if (getLinkedListData(pList, i) == toNode) {
                removeLinkedListData(pList, i);
                ret = 0;
                break;
            }
        }
    }
    else {  // شǴ    (  ).
        ret = -1;
    }
    
    return ret;
}


int removeEdgeLG(LinkedGraph* pGraph, int fromNode, int toNode)
{
    int ret = -1;
    
    ret = removeEdgeInternalLG(pGraph, fromNode, toNode);
    if (0 == ret && LINKED_GRAPH_TYPE_UNDIRECT == pGraph->graphType) {
        ret = removeEdgeInternalLG(pGraph, toNode, fromNode);
    }
    
    return ret;
}

int* getEdgeWeightLG(LinkedGraph* pGraph, int fromNode, int toNode)
{
    int *ret = NULL;
    LinkedList *pList = NULL;
    int i = 0, count = 0;
    
    if (pGraph != NULL
        && checkVertexValid(pGraph, fromNode)
        && checkVertexValid(pGraph, toNode)) {
        pList = pGraph->ppAdjEdge[fromNode];
        count = getLinkedListLength(pList);
        for(i = 0; i < count; i++) {
			LinkedListNode *pNode = getLinkedListNode(pList, i);
            if (pNode->data == toNode) {
                ret = &(pNode->weight);
                break;
            }
        }
    }
    
    return ret;
}

void displayGraphLG( LinkedGraph* pGraph)
{
    int i = 0, j = 0, count = 0;
    
    if (pGraph != NULL) {
        count = pGraph->nodeCount;
        for(i = 0; i < count; i++)	{
            for(j = 0; j < count; j++) {
				int *pWeight = getEdgeWeightLG(pGraph, i, j);
                if (pWeight != NULL) {
                    printf("%d ", *pWeight);
                }
                else {
                    printf("N ");
                }
            }
            printf("\n");
        }
    }
}

void deleteGraphLG( LinkedGraph* pGraph)
{
    int i = 0;
    
    if (pGraph != NULL) {
        for(i = 0; i < pGraph->nodeCount; i++) {
            deleteLinkedList(pGraph->ppAdjEdge[i]);
        }
        if (pGraph->ppAdjEdge != NULL) free(pGraph->ppAdjEdge);
        free(pGraph);
    }
}

int main(int argc, char *argv[])
{
    int nodeCount = 6;
    LinkedGraph *pG1 = createLinkedGraph(LINKED_GRAPH_TYPE_UNDIRECT, nodeCount);
    if (pG1 != NULL) {
        addEdgewithWeightLG(pG1, 0, 1, 1);
		addEdgewithWeightLG(pG1, 0, 2, 2);
		addEdgewithWeightLG(pG1, 1, 2, 3);
		addEdgewithWeightLG(pG1, 2, 3, 4);
		addEdgewithWeightLG(pG1, 3, 4, 5);
		addEdgewithWeightLG(pG1, 3, 5, 6);
		addEdgewithWeightLG(pG1, 4, 5, 7);
        
        printf("G1:   ׷\n");
        displayGraphLG(pG1);
        
        deleteGraphLG(pG1);
    }
    
    return 0;
}
